import os
import sys
from flask import Flask
from flask_cors import CORS

def create_app(test_config=None):
    # Create the app.
    app = Flask(__name__, instance_relative_config=True)
    CORS(app)
    print("Creating OSU Course Analytics Application: %s" % app.name)
    print("Instance path: %s" % app.instance_path)
    print("System prefix: %s" % sys.prefix)

    # Set some default configuration that the app will use.
    app.config.from_mapping(
        SECRET_KEY='dev'
    )

    if test_config is None:
        # Load the instance config, if it exists, when not testing.
        # If file is not found, we will carry on with default configuration.
        app.config.from_pyfile('config.py', silent=True)
    else:
        # Load the test config if passed in
        app.config.from_mapping(test_config)

    # Ensure the instance folder exists. In a distributed environment, this
    # will be $PREFIX/var/myapp-instance.
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass

    # a simple page that says hello
    @app.route('/hello')
    def hello():
        return 'Hello, World!'

    # Read environment variables beginning with 'OSUCA', '_' gets dropped.
    app.config.from_prefixed_env('OSUCA')
    data_source = None
    if 'DATA_SOURCE' in app.config:
        data_source = app.config['DATA_SOURCE']

    # Let's initiate the model, treating it as a db that we can query later.
    from . import db
    db.init_app(data_source)

    from . import term
    app.register_blueprint(term.bp)
    # associate the endpoint name 'index' with the / url so that
    # url_for('index') or url_for('term.index') will both work, generating the
    # same / URL either way.
    app.add_url_rule('/', endpoint='index')

    from . import arity
    app.register_blueprint(arity.bp)

    from . import combination
    app.register_blueprint(combination.bp)

    from . import summary 
    app.register_blueprint(summary.bp)

    # Use app_context() in a with block, and everything that runs in the block
    # will have access to current_app.
    from . import api
    with app.app_context():
        api.init_app()

    return app



import requests
from pkg_resources import resource_string as resource_bytes

from osuca.model.analytics import Analytics

analytics = None


def init_app(data_source):
    # Declare analytics as global to assign value to it later
    global analytics
    reviews = None
    # Let's use the static file if there is no configuration parameter passed.
    if data_source is None:
        print('Using static data source.')
        reviews = resource_bytes('osuca.static', 'reviews.csv').decode('utf-8')
        analytics = Analytics(reviews)
    else:
        reviews = requests.get(data_source)
        if reviews:
            print('Successfully retrieved %s.' % data_source)
        else:
            print('An error has occurred while retrieving %s.' % data_source)
        analytics = Analytics(reviews.text)

def get_db():
    return analytics


import pytest

from osuca import create_app

@pytest.fixture
def app():

    app = create_app({
        'TESTING': True
    })

    yield app


@pytest.fixture
def client(app):
    return app.test_client()


@pytest.fixture
def runner(app):
    return app.test_cli_runner()



from flask import current_app, g, jsonify
from flask_restful import Api, Resource

from osuca.db import get_db
from osuca.model.JSONAdapter import Encoder
from osuca.model.JSONAdapter import JSONAdapter as Adapter


def init_app():
    add_resources()


def get_api():
    if 'api' not in g:
        g.api = Api(current_app)
    return g.api


class HelloWorld(Resource):
    def get(self):
        return {'hello': 'world'}


class Course(Resource):
    def get(self):
        return jsonify(Adapter(get_db()).course())


class CourseQuarterAggregate(Resource):
    def get(self):
        return jsonify(Adapter(get_db()).course_quarter_aggregate())


class CourseYearAggregate(Resource):
    def get(self):
        return jsonify(Adapter(get_db()).course_year_aggregate())


def add_resources():
    current_app.json_encoder = Encoder
    get_api().add_resource(HelloWorld, '/restful-hello')
    get_api().add_resource(Course, '/courses')
    get_api().add_resource(CourseQuarterAggregate, '/course-quarter-aggregates')
    get_api().add_resource(CourseYearAggregate, '/course-year-aggregates')



import pytest
from osuca.db import get_db


def test_legend(client):
    response = client.get('/')
    assert b"Number of Evaluations" in response.data
    assert b"Average Course Difficulty" in response.data

def test_data(client):
    response = client.get('/')
    assert b"'mean': 2.6937799043062203" in response.data

def test_status(client):
    assert client.get('/').status_code == 200

def test_redirect(client):
    assert client.get('/term').status_code == 308 


import pytest
from osuca.db import get_db


def test_data(client):
    response = client.get('/combination/count/True')
    assert b"<td>3.09</td>" in response.data

def test_status(client):
    assert client.get('/combination/count/True').status_code == 200



from osuca import create_app


def test_config():
    assert not create_app().testing
    assert create_app({'TESTING': True}).testing


def test_hello(client):
    response = client.get('/hello')
    print(response.data)
    assert response.data == b'Hello, World!'


import pytest
from osuca.db import get_db


def test_legend(client):
    response = client.get('/arity/')
    assert b"Number of Evaluations" in response.data
    assert b"Average Course Difficulty" in response.data

def test_data(client):
    response = client.get('/arity/')
    assert b"'sum': 650" in response.data

def test_status(client):
    assert client.get('/arity/').status_code == 200



from re import M
from webbrowser import get
import requests
from flask import Blueprint, render_template, request, flash

from osuca.db import get_db

bp = Blueprint('summary', __name__)


@bp.route("/fetch", methods=['GET', 'POST'])
def fetch():
    db = get_db()
    selection_label = "All Courses"
    if request.method == 'POST' and request.form['course'] != "All Courses":
        selection_label = request.form['course']
    microservice_result = None
    try:
        response = requests.get('http://localhost:3000/stats').json()
        microservice_result = prepare(
            response['allCourseStatistics'], db.course(), selection_label)
    except:
        flash("Could not access summary service.")


    return render_template("summary.html",
                           course=sorted(db.course()),
                           microservice_result=microservice_result)


def prepare(response, course, selection_label):
    result = {}
    for r in response:
        for c in course:
            if r['course'] == c.id:
                stats = r['statistics'][0]
                ad = stats['averageDifficulty']
                ld = stats['lowestDifficulty']
                hd = stats['highestDifficulty']
                result[c.subject + ' ' + c.id + ' - ' + c.name] = (ad, ld, hd)

    if selection_label != "All Courses":
        result = {key: value for key, value in result.items() if key == selection_label}

    result = sorted(result.items(), key=lambda item: item[1][0], reverse=True)
    return result 



from flask import Blueprint, current_app, jsonify, render_template, request
from werkzeug.exceptions import abort

from osuca.db import get_db
from osuca.model.JSONAdapter import Encoder, JSONAdapter

bp = Blueprint('term', __name__)


@bp.route('/')
@bp.route("/term/", methods=['GET', 'POST'])
def index():
    db = get_db()
    adapter = JSONAdapter(db)

    query_result_json = jsonify(adapter.term_aggregate(sort=True)).get_json()
    selection_label = "All Courses"

    if request.method == 'POST' and request.form['course'] != "All Courses":
        selection_label = request.form['course']
        selection = db.course(selection_label)
        query_result_json = jsonify(
            adapter.course_term_aggregate(selection, sort=True)).get_json()

    current_app.json_encoder = Encoder
    return render_template('term.html',
                           selection_label=selection_label,
                           course=sorted(db.course()),
                           term_json=jsonify(
                               adapter.term(sort=True)).get_json(),
                           term=sorted(db.term()),
                           query_result_json=query_result_json)



import json
from enum import Enum
from json import JSONEncoder


class Encoder(JSONEncoder):
    def default(self, o):
        if isinstance(o, Term):
            return o.lookup()
        if isinstance(o, Quarter):
            return {"term": o.term.lookup(), "year": o.year}
        return json.JSONEncoder.default(self, o)


class Term(Enum):
    WINTER = 0
    SPRING = 1
    SUMMER = 2
    FALL = 3

    def lookup(self):
        dict = {
            Term.WINTER: 'Winter',
            Term.SPRING: 'Spring',
            Term.SUMMER: 'Summer',
            Term.FALL: 'Fall'
        }
        return dict[self]

    def __str__(self):
        return self.toJSON()

    def toJSON(self):
        return json.dumps(self, indent=2, cls=Encoder)

    def __lt__(self, other):
        return self.value < other.value


class Quarter:
    def __init__(self, label) -> None:
        year = ''.join(filter(str.isdigit, label))
        year = year[-2:]
        self.year = None
        if len(year) != 0 and int(year) > 14:
            self.year = '20' + year

        term = ''.join(filter(str.isalpha, label)).upper()
        self.term = None
        if 'W' in term:
            self.term = Term.WINTER
        elif 'P' in term:
            self.term = Term.SPRING
        elif 'U' in term:
            self.term = Term.SUMMER
        elif 'F' in term:
            self.term = Term.FALL

    def invalid(self):
        return self.year is None or self.term is None

    def __str__(self):
        return self.toJSON()

    def toJSON(self):
        return json.dumps(self, indent=2, cls=Encoder)

    def __key(self):
        return (self.term, self.year)

    def __hash__(self):
        return hash(self.__key())

    def __eq__(self, other):
        if isinstance(other, Quarter):
            return self.__key() == other.__key()
        return NotImplemented

    def __lt__(self, other):
        if self.year < other.year:
            return True
        elif self.year > other.year:
            return False
        else:
            return self.term < other.term


if __name__ == '__main__':
    q1 = Quarter('f2019')
    q2 = Quarter('w2019')
    q3 = Quarter('u2018')
    q4 = Quarter('p2015')
    q5 = Quarter('u2019')
    q6 = Quarter('p2019')
    print(q1)
    print(q2)
    print(q3)
    print(q4)

    s = {q6, q5, q2, q3, q4, q1}
    sorted_list = sorted(list(s))
    print(json.dumps(sorted_list, indent=2, cls=Encoder))

    t1 = Term.WINTER
    t2 = Term.FALL
    t3 = Term.SPRING
    s = {t1, t2, t3}
    sorted_list = sorted(list(s))
    print(json.dumps(sorted_list, indent=2, cls=Encoder))
    print(json.dumps(t1, indent=2, cls=Encoder))
    print(t1)
